﻿// CarConnection Class
//Code for the Power Split Device demo
//Author: Alex Hart http://eahart.com
//Contributing author: J. Kusnetz http://www.kusnetz.net/prius/

/*

Copyright (c) 2006, Alex Hart
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
    * Neither the name of Alex Hart nor the names of its
	contributors may be used to endorse or promote products derived
	from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

class CarConnection {
	//Private vars
	private var mDoCanView : Boolean = false;
	private var mPSDlocals : String = "PSD_drive_locals.txt";
	private var mUpdateSpeed : Number = 3;
	private var iNum : Number = 0;
	private var mCarMode : Boolean = false;
	private var mXmlTestMode : Boolean = false;

	//public vars
	public static var mParent : MovieClip;
	public static var mThis : Object;
	public var myXMLSocket : XMLSocket;
	public var myLoadVars : LoadVars;
	
	//constructor
	public function CarConnection( pTargetClip : MovieClip )
	{
		mParent = pTargetClip;
		mThis = this;

		myXMLSocket = new XMLSocket();
		myLoadVars = new LoadVars();
		myLoadVars.load( mPSDlocals );
	
		var myMC : MovieClip = mParent.createEmptyMovieClip("XMLLoop_mc", 999);
		//run this function every frame
		myMC.onEnterFrame = CanView;

		myLoadVars.onLoad = function ()
		{
			if (this.CarMode)
			{
				mThis.setCarMode ( this.CarMode );			
				mThis.setCanView ( true );
				mParent.MG2toMPHConversionConstant = this.MG2toMPHConversionConstant;
				trace("Entering Car Mode...");
				
				mThis.myXMLSocket.onConnect = mThis.mySocketConnect;
				mThis.myXMLSocket.connect(this.myHost,this.myPort);
				
				mParent.gotoAndPlay("CarMode");
			}
		}
	}

	/////////////////////////////
	//this function is used external to the class, so needs path references to the class object
	private function CanView ()
	{ 
		if ( mThis.mDoCanView )
		{
			//use a frame counter to test when to run
			if ( mThis.getI() == mThis.getUpdateSpeed() )
			{
				//if we're in fake car mode
				if ( mThis.getXmlTestMode() )
				{
					//fake the XML
					var lMPG : Number = (Math.sin( getTimer()/2000 ) + 1)*127.5;
					var lSOC : Number = ((Math.sin( getTimer()/1600 ) + 1)*20 ) + 40;
					var lMG2RPM : Number = ((Math.sin( getTimer()/2200 ) + 1)*3850) - 1200;
					var lICERPM : Number = (Math.sin( getTimer()/2100 ) + 1)*2250;
					EngineAttrs.MG2RPM = lMG2RPM;
					EngineAttrs.ICERPM = lICERPM;
					EngineAttrs.MPG    = lMPG;
					EngineAttrs.SOC    = lSOC;
					mParent.Engine.updateMG1();
					mThis.setI( 0 );
					//trace(lMPG+", "+lSOC+", "+lMG2RPM+", "+lICERPM);
				} else {
					//otherwise, go get the real data from the car
					mThis.updateXML();					
					mThis.setI( 0 );
				}
			}
			mThis.setI( mThis.getI() + 1 );
		}
	}
	
	private function mySocketConnect ( pBool : Boolean )
	{
		//if the XML socket connects successfully
		if ( pBool ) {
			//scale root, and set nomo scale too
			mParent._yscale = 131.25;
			DrawNomograph.setScaleFactor ( 1.3125 );
		} else
		{
			//otherwise, leave the scale alone, and turn on fake XML mode
			mThis.setXmlTestMode( true );
		}
	}
	
	public function updateXML() : Void
	{
				//trace("Updating XML...");
				var myXML : XML = new XML();
				mThis.myXMLSocket.send(myXML);
				mThis.myXMLSocket.onXML = myOnXML;
	}

	private function myOnXML (doc)
	{
		var e = doc.firstChild;
		if (e != null && e.nodeName == "MESSAGE")
		{
			EngineAttrs.MG2RPM  = e.attributes.MG2RPM;
			EngineAttrs.ICERPM  = e.attributes.ICERPM;
			EngineAttrs.MPG     = e.attributes.MPG;
			EngineAttrs.SOC     = e.attributes.SOC;
		}
	}

	public function setCarMode ( pBool : Boolean ) : Void
	{
		mCarMode = pBool;
	}
	public function getCarMode () : Boolean
	{
		return mCarMode;
	}
	public function setCanView ( pBool : Boolean ) : Void
	{
		mDoCanView = pBool;
	}
	public function getCanViewStatus () : Boolean
	{
		return mDoCanView;
	}

	public function setI ( pI ) : Void
	{
		iNum = pI;
	}
	public function getI () : Number
	{
		return iNum;		
	}
	public function getUpdateSpeed () : Number
	{
		return mUpdateSpeed;
	}
	public function setXmlTestMode ( pBool : Boolean ) : Void
	{
		mXmlTestMode = pBool;
	}
	public function getXmlTestMode () : Boolean
	{
		return mXmlTestMode;
	}
	
	public function showVars () : String
	{
		return mPSDlocals + ", " + mDoCanView + ", " + mParent + ", " + mThis + ", " + mUpdateSpeed;
	}
}